package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysTaskTimeExpressionService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysTaskTimeService;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysRefExpressionTrigger;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskTime;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskTimeExpression;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description : 系统时间表达式 服务
 * @Reference   :
 * @Author      : wei.peng
 * @Date        : 2018-10-22 16:58:43.838
 * @Modify      :
 **/
@RestController
@Api(description = "系统时间表达式")
@RequestMapping(PlatformConstWords.BASE_URL +"/task-time-expression")
public class SysTaskTimeExpressionController extends CoreBaseController{

    public static final Logger LOGGER = LoggerFactory.getLogger(SysTaskTimeExpressionController.class);

    @Autowired
    private ISysTaskTimeExpressionService taskTimeExpressionService;

    @Autowired
    private ISysTaskTimeService sysTaskTimeService;

    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增系统时间表达式", notes = "系统时间表达式")
    public ResultBean insertSysTaskTimeExpression(SysTaskTimeExpression timeExpression) {
        try {
            // 数据校验
            ValidatorBean.beginValid(timeExpression)
                    .notNull("name", timeExpression.getName())
                    .notNull("expressionContent", timeExpression.getExpressionContent())
                    .notNull("expressionStartDatetime",timeExpression.getExpressionStartDatetime());

            taskTimeExpressionService.insertSysTaskTimeExpression(timeExpression);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping(value = "/update")
    @ApiOperation(value = "修改系统时间表达式", notes = "系统时间表达式")
    public ResultBean updateSysTaskTimeExpression(SysTaskTimeExpression timeExpression) {
        try {
            // 数据校验
            ValidatorBean.beginValid(timeExpression)
                    .notNull("id", timeExpression.getId())
                    .notNull("name", timeExpression.getName())
                    .notNull("timeExpression", timeExpression.getExpressionContent());

            taskTimeExpressionService.updateSysTaskTimeExpression(timeExpression);

            // 查询关联定时任务信息，并进行修改
            List<SysRefExpressionTrigger> refTrigger = taskTimeExpressionService.findRefExpressionTriggerList(timeExpression.getId());
            SysTaskTime sysTaskTime;
            for (SysRefExpressionTrigger item: refTrigger) {
                sysTaskTime = sysTaskTimeService.getSysTaskTimeByNameAndGroup(item.getTriggerName(),item.getTriggerGroupName());
                sysTaskTime.setTimeExpressionNameRdd(timeExpression.getName());
                sysTaskTime.setTimeExpressionContentRdd(timeExpression.getExpressionContent());
                sysTaskTime.setTaskStartDateTimeRdd(timeExpression.getExpressionStartDatetime());
                sysTaskTime.setTaskEndDateTimeRdd(timeExpression.getExpressionEndDatetime());

                sysTaskTimeService.updateSysTaskTime(sysTaskTime,timeExpression);
            }

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "删除系统时间表达式", notes = "删除系统时间表达式")
    public ResultBean deleteSysTaskTimeExpression(@PathVariable("id") String idStr) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(idStr, "时间表达式id 不能为空");
            if(taskTimeExpressionService.getSysTaskTimeExpressionById(Long.parseLong(idStr)) == null){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                        .setErrorDetail("时间表达式不存在")
                        .setErrorSolution("请重新选择")
                        .build();
            }

            taskTimeExpressionService.deleteSysTaskTimeExpressionById(Long.parseLong(idStr));
            return ResultBean.success().setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/list")
    @ApiOperation(value = "查询时间表达式所有", notes = "查询所有系统时间表达式")
    public ResultBean findSysTaskTimeExpressionAll() {
        try {
            List<SysTaskTimeExpression> list = taskTimeExpressionService.listSysTaskTimeExpression();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(list);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "查询系统时间表达式", notes = "查询根据ID查询系统时间表达式")
    public ResultBean getSysTaskTimeExpressionById(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "定时任务id 不能为空");

            SysTaskTimeExpression taskTimeExpression = taskTimeExpressionService.getSysTaskTimeExpressionById(Long.parseLong(id));

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(taskTimeExpression);
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping("query")
    @ApiOperation(value = "时间表达式查询，分页，排序",notes = "时间表达式查询，分页，排序")
    public ResultBean querySysTaskTimeExpressionByPager(SysTaskTimeExpression taskTimeExpression, Pager pager){
        try {
            ListPager taskTimeExpressionListPager = taskTimeExpressionService.querySysTaskTimeExpressionByPager(taskTimeExpression,pager);
            return  ResultBean.success("查询成功").setListPager(taskTimeExpressionListPager).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @DeleteMapping("/batch-delete")
    @ApiOperation(value = "根据id批量删除时间表达式",notes = "根据id批量删除时间表达式")
    public ResultBean deleteSysTaskTimeExpressionByIds(String[] idsStr){
        try{
            if(idsStr.length == 0){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的资源。")
                        .build();
            }

            taskTimeExpressionService.deleteSysTaskTimeExpressionByIds(StringTool.getArrayLong(idsStr));
            return ResultBean.success("删除成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
