package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.core.api.iservice.busi.ISysTaskTimeService;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskTime;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogTaskTime;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;

/**
 * @Description : 定时任务通道业务处理
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-12-20 21:56
 * @Modify:
 **/
@Component
public class ScheduleQueueReceiver {

    private static final Logger LOGGER = LoggerFactory.getLogger(ScheduleQueueReceiver.class);

    @Autowired
    private ISysTaskTimeService sysTaskTimeService;

    /**
     * 定时任务通道处理
     * @param logTaskTime
     * @param channel
     * @param message
     */
//    @RabbitListener(queues = PlatformConstWords.IMPP_SCHEDULE_QUEUE)
    public void processSchedule(SysLogTaskTime logTaskTime, Channel channel, Message message) {
        try {
            LOGGER.info("【MQ-IMPP_SCHEDULE_QUEUE】数据接收成功：{}",logTaskTime);
            // 跟新最后执行时间 及 任务状态
            SysTaskTime task = sysTaskTimeService.getSysTaskTimeByNameAndGroup(logTaskTime.getName(),logTaskTime.getGroupName());
            task.setLastRunDateTime(logTaskTime.getCreateDatetime());
            task.setTaskStatus(logTaskTime.getTaskStatus());
            sysTaskTimeService.updateSysTaskTime(task);

            // 添加定时任务日志
            logTaskTime.setTimeTaskId(task.getId());
            logTaskTime.setTimeExpressionNameRdd(task.getTimeExpressionNameRdd());
            logTaskTime.setTimeExpressionContentRdd(task.getTimeExpressionContentRdd());
            sysTaskTimeService.logInsertSysLogTaskTime(logTaskTime);

            //信息已处理
            channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
        } catch (IOException e) {
            LOGGER.error("【MQ-IMPP_MESSAGE_MAIL_QUEUE】处理出错：{}",e.getMessage(),e);
            //丢弃这条消息
            try {
                // 未成功处理，重新发送
                channel.basicNack(message.getMessageProperties().getDeliveryTag(),false,true);
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
    }
}
