package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.IPersonnelService;
import cn.estsh.i3plus.core.apiservice.util.HqlModelPack;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.shirotoken.UserToken;
import cn.estsh.i3plus.pojo.base.tool.HqlPack;
import cn.estsh.i3plus.pojo.model.platform.UserDetailModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.i3plus.pojo.platform.repository.*;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author : Adair Peng
 * @CreateDate : 2018-12-24 13:10
 * @Modify:
 **/
@Service
public class PersonnelServiceService implements IPersonnelService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysConfigService.class);

    @Autowired
    private SysUserRepository userRDao;

    @Autowired
    private SysUserInfoRepository userInfoRDao;

    @Autowired
    private SysRoleRepository roleRDao;

    @Autowired
    private SysRefUserDepartmentRepository refUserDepartmentRDao;

    @Autowired
    private SysRefUserInfoDepartmentRepository refUserInfoDepartmentRDao;

    @Autowired
    private SysRefUserPositionRepository refUserPositionRDao;

    @Autowired
    private SysRefUserRoleRepository refUserRoleRDao;

    @Autowired
    private SysOrganizeRepository organizeRDao;

    @Autowired
    private SysDepartmentRepository departmentRDao;

    @Autowired
    private SysPositionRepository positionRDao;


    @Override
    public SysUser saveSysUser(SysUser user) {
        LOGGER.debug("平台账号 SysUser user:{}", user);
        return userRDao.save(user);
    }

    @Override
    public SysUserInfo saveSysUserInfo(SysUserInfo info) {
        LOGGER.debug("平台用户 SysUserInfo info:{}", info);
        return userInfoRDao.save(info);
    }

    @Override
    public void updateSysUserInfoStatus(Long userInfoId, Integer status) {

    }

    @Override
    @ApiOperation(value = "删除账号信息")
    public void deleteSysUser(Long[] userIdList,String userName) {
        LOGGER.debug("平台账号 SysUser userIdList:{}", userIdList);
        // 删除账号信息
        userRDao.updateValidStatusByPropertyIn("id", userIdList,CommonEnumUtil.IS_VAILD.INVAILD.getValue(), userName);

        // 删除账号关系
        refUserDepartmentRDao.deleteByPropertyIn("userId",userIdList);
        refUserRoleRDao.deleteByPropertyIn("userId", userIdList);
    }

    @Override
    @ApiOperation(value = "删除用户信息")
    public void deleteSysUserInfo(Long[] infoIdList,String userName) {
        LOGGER.debug("平台用户 SysUserInfo infoIdList:{}", infoIdList);
        // 删除用户信息
        userInfoRDao.updateValidStatusByPropertyIn("id", infoIdList,CommonEnumUtil.IS_VAILD.INVAILD.getValue(), userName);

        // 删除用户关系
        refUserInfoDepartmentRDao.deleteByPropertyIn("userId",infoIdList);
        refUserPositionRDao.deleteByPropertyIn("userId",infoIdList);
    }

    @Override
    public SysUser getSysUserById(Long id) {
        return userRDao.getById(id);
    }

    @Override
    public List<SysUser> findSysUserByIds(Long[] ids) {
        LOGGER.debug("平台用户 SysUserInfo ids:{}", ids);

        String infoWhere = CoreHqlPack.packHqlIds("id", ids);
        return userRDao.findByHqlWhere(infoWhere);
    }

    @Override
    public List<SysUser> querySysUserByUserDetailModelOrIds(UserDetailModel model, List<Long> ids) {
        LOGGER.debug("平台用户 SysUser model:{} ids:{}", model, ids);
        String modeWhere = HqlModelPack.packHqlSysUserByModelOrIds(model, ids);

        return userRDao.findByHqlWhere(modeWhere);
    }

    @Override
    public SysUserInfo getSysUserInfoById(Long id) {
        return userInfoRDao.getById(id);
    }

    @Override
    public SysDepartment getSysDepartmentById(Long id) {
        return departmentRDao.getById(id);
    }

    @Override
    public SysPosition getSysPositionById(Long id) {
        return positionRDao.getById(id);
    }

    @Override
    public SysOrganize getSysOrganizeById(Long id) {
        return organizeRDao.getById(id);
    }

    @Override
    public SessionUser queryUserLogin(String userName, String password, String languageCode) {
        LOGGER.debug("平台用户 SYS_USER loginName:{} \t password:{} \t languageCode:{}", userName, password, languageCode);
        UserToken token = new UserToken(userName,password,languageCode);
        SessionUser sessionUser = AuthUtil.login(token);
        AuthUtil.setSessionUser(sessionUser);
        return sessionUser;
    }

    @Override
    public ListPager<SysUserInfo> querySysUserInfo(UserDetailModel model, Pager pager) {
        return null;
    }

    @Override
    public List<SysUserInfo> findSysUserInfoByIds(Long[] ids) {
        LOGGER.debug("平台用户 SysUserInfo ids:{}", ids);

        String infoWhere = CoreHqlPack.packHqlIds("id", ids);
        return userInfoRDao.findByHqlWhere(infoWhere);
    }

    @Override
    public List<SysRefUserRole> findSysRefUserRoleById(Long userId) {
        LOGGER.debug("平台账号角色 SysRefUserRole userId:{}", userId);
        return refUserRoleRDao.findByProperty("userId",userId);
    }

    @Override
    public List<SysRefUserDepartment> findSysRefUserDepartmentById(Long userId) {
        LOGGER.debug("平台账号部门 SysRefUserDepartment userId:{}", userId);
        return refUserDepartmentRDao.findByProperty("userId",userId);
    }

    @Override
    public List<SysRefUserDepartment> querySysRefUserDepartmentByUserDetailModel(UserDetailModel model) {
        if(model != null){
            StringBuffer userIdWhere = new StringBuffer();

            if(model.getUserDepartmentIdList() != null){
                HqlPack.getInPack(StringUtils.join(model.getInfoDepartmentIdList(),","),"departmentId", userIdWhere);
            }
            if(model.getUserOrganizeIdList() != null){
                HqlPack.getInPack( StringUtils.join(model.getUserOrganizeIdList(),","),"organizeId",userIdWhere);
            }

            return refUserDepartmentRDao.findByHqlWhere(userIdWhere.toString());
        }else {
            return null;
        }
    }

    @Override
    public List<SysRefUserInfoDepartment> findSysRefUserInfoDepartmentById(Long infoId) {
        LOGGER.debug("平台用户部门关系 SysRefUserInfoDepartment infoId:{}", infoId);
        return refUserInfoDepartmentRDao.findByProperty("userId",infoId);
    }

    @Override
    @ApiOperation(value = "查询用户部门关系")
    public List<SysDepartment> findSysUserInfoDepartmentByInfoId(Long infoId) {
        LOGGER.debug("平台用户部门 SysDepartment infoId:{}", infoId);
        List<SysDepartment> result = new ArrayList<>();

        List<SysRefUserInfoDepartment> refs = findSysRefUserInfoDepartmentById(infoId);
        if(refs != null && refs.size() > 0){
            Long[] ids = new Long[refs.size()];
            for (int i = 0; i < refs.size(); i++) {
                ids[i] = refs.get(i).getDepartmentId();
            }
            String idWhere = CoreHqlPack.packHqlIds("id", ids);
            return departmentRDao.findByHqlWhere(idWhere);
        }
        return result;
    }

    @Override
    @ApiOperation(value = "查询账号部门关系")
    public List<SysDepartment> findSysUserDepartmentByInfoId(Long infoId) {
        LOGGER.debug("平台用户部门 SysDepartment infoId:{}", infoId);
        List<SysDepartment> result = new ArrayList<>();

        List<SysRefUserDepartment> refs = findSysRefUserDepartmentById(infoId);
        if(refs != null && refs.size() > 0){
            Long[] ids = new Long[refs.size()];
            for (int i = 0; i < refs.size(); i++) {
                ids[i] = refs.get(i).getDepartmentId();
            }
            String idWhere = CoreHqlPack.packHqlIds("id", ids);
            return departmentRDao.findByHqlWhere(idWhere);
        }
        return result;
    }

    @Override
    public List<SysOrganize> findSysUserInfoOrganize(Long infoId) {
        LOGGER.debug("平台用户部门 SysDepartment infoId:{}", infoId);
        List<SysOrganize> result = new ArrayList<>();

        List<SysRefUserInfoDepartment> refs = findSysRefUserInfoDepartmentById(infoId);
        if(refs != null && refs.size() > 0){
            Long[] ids = new Long[refs.size()];
            for (int i = 0; i < refs.size(); i++) {
                ids[i] = refs.get(i).getOrganizeId();
            }
            String idWhere = CoreHqlPack.packHqlIds("id", ids);
            return organizeRDao.findByHqlWhere(idWhere);
        }
        return result;
    }

    @Override
    public List<SysOrganize> findSysUserOrganize(Long infoId) {
        LOGGER.debug("平台用户部门 SysDepartment infoId:{}", infoId);
        List<SysOrganize> result = new ArrayList<>();

        List<SysRefUserDepartment> refs = findSysRefUserDepartmentById(infoId);
        if(refs != null && refs.size() > 0){
            Long[] ids = new Long[refs.size()];
            for (int i = 0; i < refs.size(); i++) {
                ids[i] = refs.get(i).getOrganizeId();
            }
            String idWhere = CoreHqlPack.packHqlIds("id", ids);
            return organizeRDao.findByHqlWhere(idWhere);
        }
        return result;
    }

    @Override
    public List<SysRefUserPosition> findSysRefUserPositionById(Long infoId) {
        LOGGER.debug("平台用户岗位 SysRefUserPosition infoId:{}", infoId);
        return refUserPositionRDao.findByProperty("userId",infoId);
    }

    @Override
    public SessionUser packSessionUser(SessionUser sessionUser, SysUser user, Integer userType, String languageCode) {
        return null;
    }

    @Override
    public void checkSysUserOnly(SysUser user) {
        if(user != null){
            // 登录名唯一教研
            if(!checkSysUserLoginName(user.getUserLoginName(), user.getId() == null ? 0 : user.getId())){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                        .setErrorDetail("用户名已存在")
                        .setErrorSolution("请重新输入用户名")
                        .build();
            }

            // 邮箱唯一教研
            if(!checkSysUserEmail(user.getUserEmail(),user.getUserInfoId() == null ? 0 : user.getUserInfoId())){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                        .setErrorDetail("邮箱已存在")
                        .setErrorSolution("请重新输入邮箱")
                        .build();
            }

            // 手机号唯一教研
            if(!checkSysUserPhone(user.getUserPhone(),user.getUserInfoId() == null ? 0 : user.getUserInfoId())){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                        .setErrorDetail("手机号已存在")
                        .setErrorSolution("请重新输入手机号")
                        .build();
            }

            // 工号唯一教研
            if(!checkSysUserEmpNo(user.getUserEmpNo(),user.getUserInfoId() == null ? 0 : user.getUserInfoId())){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                        .setErrorDetail("工号号已存在")
                        .setErrorSolution("请重新输入工号")
                        .build();
            }
        }
    }

    /************************************ 用户唯一检查 ****************************************/

    /**
     * 登录名 唯一检查
     * @param userLoginName
     * @param userId
     * @return
     */
    private boolean checkSysUserLoginName(String userLoginName, long userId) {
        int count = 0;
        if(userId > 0){
            StringBuffer sw = new StringBuffer();
            HqlPack.getNumEqualPack(CommonEnumUtil.IS_VAILD.VAILD.getValue(),"isValid",sw);
            HqlPack.getStringEqualPack(userLoginName,"userLoginName",sw);
            HqlPack.getNumNOEqualPack(userId,"id",sw);
            count = userRDao.findByHqlWhereCount(sw.toString());

        }else{
            count = userRDao.findByPropertyCount(
                    new String[]{"userLoginName","isValid"},
                    new Object[]{userLoginName,CommonEnumUtil.IS_VAILD.VAILD.getValue()});
        }
        return count <= 0 ;
    }

    /**
     * 用户邮箱唯一检查
     * @param email
     * @param infoId
     * @return
     */
    private boolean checkSysUserEmail(String email, long infoId) {
        int count;
        if(infoId > 0){
            StringBuffer sw = new StringBuffer();
            HqlPack.getNumEqualPack(CommonEnumUtil.IS_VAILD.VAILD.getValue(),"isValid",sw);
            HqlPack.getStringEqualPack(email,"userEmail",sw);
            HqlPack.getNumNOEqualPack(infoId,"userInfoId",sw);
            count = userRDao.findByHqlWhereCount(sw.toString());

        }else{
            count = userRDao.findByPropertyCount(
                    new String[]{"userEmail","isValid"},
                    new Object[]{email,CommonEnumUtil.IS_VAILD.VAILD.getValue()});
        }
        return count <= 0 ;
    }

    /**
     * 手机号码 唯一校验
     * @param phone
     * @param infoId
     * @return
     */
    private boolean checkSysUserPhone(String phone, long infoId) {
        int count;
        if(infoId > 0){
            StringBuffer sw = new StringBuffer();
            HqlPack.getNumEqualPack(CommonEnumUtil.IS_VAILD.VAILD.getValue(),"isValid",sw);
            HqlPack.getStringEqualPack(phone,"userPhone",sw);
            HqlPack.getNumNOEqualPack(infoId,"userInfoId",sw);
            count = userRDao.findByHqlWhereCount(sw.toString());

        }else{
            count = userRDao.findByPropertyCount(
                    new String[]{"userPhone","isValid"},
                    new Object[]{phone,CommonEnumUtil.IS_VAILD.VAILD.getValue()});
        }
        return count <= 0 ;
    }

    /**
     * 工号唯一检查
     * @param empNo
     * @param infoId
     * @return
     */
    private boolean checkSysUserEmpNo(String empNo, long infoId) {
        int count;
        if(infoId > 0){
            StringBuffer sw = new StringBuffer();
            HqlPack.getNumEqualPack(CommonEnumUtil.IS_VAILD.VAILD.getValue(),"isValid",sw);
            HqlPack.getStringEqualPack(empNo,"userEmpNo",sw);
            HqlPack.getNumNOEqualPack(infoId,"userInfoId",sw);
            count = userRDao.findByHqlWhereCount(sw.toString());

        }else{
            count = userRDao.findByPropertyCount(
                    new String[]{"userEmpNo","isValid"},
                    new Object[]{empNo,CommonEnumUtil.IS_VAILD.VAILD.getValue()});
        }
        return count <= 0 ;
    }

    @Override
    public void refreshRefSysUserOrganizeRdd(Long organizeIds) {

    }

    @Override
    public void refreshRefSysUserDepartment(Long userId, Long[] departmentIds) {
        LOGGER.info("账号部门信息 SysUser userInfoId:{} departmentIds:{}",userId,departmentIds);

        SysUser user = userRDao.getById(userId);
        if(user != null ){
            // 删除用户部门关系
            refUserDepartmentRDao.deleteByProperty("userId",user.getId());

            if(departmentIds.length > 0){
                // 查询部门信息
                StringBuffer where = new StringBuffer();
                HqlPack.getInPack(StringUtils.join(departmentIds,","),"id",where);
                List<SysDepartment> departmentList = departmentRDao.findByHqlWhere(where.toString());

                if(departmentList != null && departmentList.size() >0){
                    List<SysRefUserDepartment> list = new ArrayList<>();
                    List<String> nameList = new ArrayList<>();
                    SysRefUserDepartment ref = null;
                    for (SysDepartment department : departmentList) {
                        ref = new SysRefUserDepartment();
                        ref.setOrganizeId(department.getOrganizeId());
                        ref.setOrganizeNameRdd(department.getOrganizeNameRdd());
                        ref.setDepartmentId(department.getId());
                        ref.setDepartmentNameRdd(department.getName());
                        ref.setUserId(user.getId());
                        ref.setUserNameRdd(user.getUserName());
                        list.add(ref);

                        nameList.add(department.getName());
                    }

                    user.setDepartmentNamesRdd(String.join(",",nameList));

                    refUserDepartmentRDao.saveAll(list);
                    userRDao.save(user);

                    LOGGER.info("账号部门信息 SysUserInfo RefUserDepartmentRDao Insert Size :{}",list.size());
                }
            }
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("用户信息不存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }
    }

    @Override
    public void refreshRefSysUserRole(Long userId, Long[] roleIds) {
        LOGGER.debug("账号角色关系信息 RefSysUserRole userId:{} roleIds:{} ", userId,roleIds);
        SysUser user = userRDao.getById(userId);
        if(user != null){
            // 根据IDS 查询所有的角色信息
            String where = CoreHqlPack.packHqlIds("id", roleIds);
            List<SysRole> list = roleRDao.findByHqlWhere(where);

            // 删除角色关系信息
            refUserRoleRDao.deleteByProperty("userId", user.getId()); // 删除用户角色关系

            if(null != list && list.size() > 0){
                // 新增用户角色关系
                List<SysRefUserRole> refUserRoles = new ArrayList<>();
                List<String> roleNameList = new ArrayList<>();
                SysRefUserRole refUserRole = null;
                for (SysRole role : list) {
                    roleNameList.add(role.getName());
                    // 用户角色关系创建
                    refUserRole = new SysRefUserRole();
                    refUserRole.setRoleId(role.getId());
                    refUserRole.setRoleNameRdd(role.getName());
                    refUserRole.setUserId(user.getId());
                    refUserRole.setUserNameRdd(user.getUserName());
                    refUserRole.setIsValid(CommonEnumUtil.IS_VAILD.VAILD.getValue());
                    refUserRoles.add(refUserRole);
                }
                refUserRoleRDao.saveAll(refUserRoles);

                // 角色名称冗余
                user.setRoleNamesRdd(String.join(",", roleNameList));

                userRDao.save(user);
            }
        }
    }

    @Override
    public void refreshRefSysUserDepartmentRdd(Long departmentId) {

    }

    @Override
    public void refreshRefSysUserInfoDepartment(Long userInfoId, Long[] departmentIds) {
        LOGGER.info("用户部门关系信息 SysUser userInfoId:{} departmentIds:{}",userInfoId,departmentIds);

        SysUserInfo info = userInfoRDao.getById(userInfoId);
        if(info != null ){
            // 删除用户部门关系
            refUserInfoDepartmentRDao.deleteByProperty("userId",info.getId());

            if(departmentIds.length > 0){
                // 查询部门信息
                StringBuffer where = new StringBuffer();
                HqlPack.getInPack(StringUtils.join(departmentIds,","),"id",where);
                List<SysDepartment> departmentList = departmentRDao.findByHqlWhere(where.toString());

                if(departmentList != null && departmentList.size() >0){
                    List<SysRefUserInfoDepartment> list = new ArrayList<>();
                    List<String> nameList = new ArrayList<>();
                    SysRefUserInfoDepartment ref = null;
                    for (SysDepartment department : departmentList) {
                        ref = new SysRefUserInfoDepartment();
                        ref.setOrganizeId(department.getOrganizeId());
                        ref.setOrganizeNameRdd(department.getOrganizeNameRdd());
                        ref.setDepartmentId(department.getId());
                        ref.setDepartmentNameRdd(department.getName());
                        ref.setUserId(info.getId());
                        ref.setUserNameRdd(info.getName());
                        list.add(ref);

                        nameList.add(department.getName());
                    }

                    info.setDepartmentNamesRdd(String.join(",",nameList));

                    refUserInfoDepartmentRDao.saveAll(list);
                    userInfoRDao.save(info);

                    LOGGER.info("用户部门关系信息 SysUserInfo RefUserDepartmentRDao Insert Size :{}",list.size());
                }
            }
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("用户信息不存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }
    }

    @Override
    public void refreshRefSysUserInfoDepartmentRdd(Long departmentId) {

    }

    @Override
    public void refreshRefSysUserInfoPosition(Long userInfoId, Long[] positionIds) {
        LOGGER.info("用户岗位关系信息 RefSysUserInfoPosition userInfoId:{} positionIds:{} userName:{}",userInfoId,positionIds);

        SysUserInfo userInfo = userInfoRDao.getById(userInfoId);
        if(userInfo != null ){
            // 删除用户岗位关系
            refUserPositionRDao.deleteByProperty("userId",userInfo.getId());

            if(positionIds.length > 0){
                StringBuffer where = new StringBuffer();
                HqlPack.getInPack(StringUtils.join(positionIds,","),"id",where);  // 查询部门信息
                List<SysPosition> positionList = positionRDao.findByHqlWhere(where.toString());

                if(positionList != null && positionList.size() >0){
                    List<SysRefUserPosition> list = new ArrayList<>();
                    List<String> nameList = new ArrayList<>();
                    SysRefUserPosition ref = null;
                    for (SysPosition position : positionList) {
                        ref = new SysRefUserPosition();
                        ref.setUserId(userInfo.getId());
                        ref.setUserNameRdd(userInfo.getName());
                        ref.setPositionId(position.getId());
                        ref.setPositionNameRdd(position.getName());
                        list.add(ref);

                        nameList.add(position.getName());
                    }

                    userInfo.setPositionNamesRdd(String.join(",",nameList));

                    refUserPositionRDao.saveAll(list);
                    userInfoRDao.save(userInfo);

                    LOGGER.info("用户岗位关系信息 SysUserInfo RefUserPositionRDao Insert Size :{}",list.size());
                }
            }
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("用户信息不存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }
    }

    @Override
    public void refreshRefSysUserInfoPositionRdd(Long positionId) {

    }
}
