package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysRoleService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.HqlPack;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysMenu;
import cn.estsh.i3plus.pojo.platform.bean.SysRefRoleMenu;
import cn.estsh.i3plus.pojo.platform.bean.SysRole;
import cn.estsh.i3plus.pojo.platform.repository.SysMenuRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysRefRoleMenuRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysRoleRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 系统角色业务接口实现
 * @Reference   :
 * @Author      : wei.peng
 * @Date        : 2018-10-22 16:58:43.779
 * @Modify      :
 **/
@Service
public class SysRoleService implements ISysRoleService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysRoleService.class);

    @Autowired
    private SysRoleRepository sysRoleRDao;

    @Autowired
    private SysMenuRepository sysMenuRDao;

    @Autowired
    private SysRefRoleMenuRepository refRoleMenuRDao;

    @Override
    @ApiOperation(value = "更新角色信息",notes = "更新角色信息")
    public void updateSysRole(SysRole sysRole) {
        LOGGER.info("系统角色 SYS_ROLE :{}", sysRole);
        sysRoleRDao.update(sysRole);
    }

    @Override
    @ApiOperation(value = "更新角色信息",notes = "更新角色状态信息")
    public void updateSysRoleStatus(Long id, int status, SessionUser user) {
        LOGGER.info("系统角色 SYS_MENU  id:{} status:{} user:{}", id,status,user);
        SysRole role = sysRoleRDao.getById(id);
        role.setRoleStatus(status);
        role.setModifyUser(user.getUserName());
        sysRoleRDao.update(role);
    }

    @Override
    @ApiOperation(value = "更新角色信息",notes = "根据ID 批量更新角色状态信息")
    public void updateSysRoleStatusByIds(Long[] ids, int status, SessionUser user) {
        LOGGER.info("系统功能 SYS_MENU  ids:{}  status:{} user:{}", ids, status, user);
        StringBuffer where = new StringBuffer();

        HqlPack.getInPack(StringUtils.join( ids,","), "id", where);
        sysRoleRDao.updateByHqlWhere(where.toString(), "roleStatus", status);
    }

    @Override
    @ApiOperation(value = "删除角色信息",notes = "根据ID 删除角色信息")
    public void deleteSysRoleById(Long id) {
        LOGGER.info("系统角色 SYS_ROLE Key:{}", id);
        refRoleMenuRDao.deleteByProperty("roleId",id);
        sysRoleRDao.deleteById(id);
    }

    @Override
    @ApiOperation(value = "删除角色信息",notes = "根据ID 批量删除角色信息")
    public void deleteSysRoleByIds(Long[] ids) {
        LOGGER.info("系统角色 SYS_ROLE ids :{}", ids);
        long positionCount = sysRoleRDao.findByHqlWhereCount(CoreHqlPack.packHqlIds("parentId",ids));
        if (positionCount >= 1) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("存在相关系统功能信息无法删除！")
                    .setErrorSolution("请先删除子集信息在操作")
                    .build();
        }else {
            refRoleMenuRDao.deleteByPropertyIn("roleId",ids);
            sysRoleRDao.deleteByIds(ids);
        }
    }

    @Override
    @ApiOperation(value = "新增角色信息")
    public SysRole insertSysRole(SysRole sysRole) {
        LOGGER.info("系统角色 SYS_ROLE :{}", sysRole);
        return sysRoleRDao.insert(sysRole);
    }

    @Override
    @ApiOperation(value = "刷新角色菜单关系",notes = "刷新 角色 菜单关系信息")
    public void refreshSysRoleRef(Long roleId,Long[] ids,String userName) {
        LOGGER.info("系统角色 SYS_ROLE :{}", roleId);
        LOGGER.info("系统角色 List<SysMenu> :{}", ids);

        refreshBatchSysRoleRef(new Long[]{roleId}, ids, userName);
    }

    @Override
    @ApiOperation(value = "刷新角色菜单关系",notes = "批量刷新 角色 菜单关系信息")
    public void refreshBatchSysRoleRef(Long[] roleIds, Long[] ids,String userName) {
        LOGGER.info("系统角色 String[] :{}", roleIds);
        LOGGER.info("系统角色 List<SysMenu> :{}", ids);

        String roleWhere = CoreHqlPack.packHqlIds("id", roleIds);
        List<SysRole> roleList = sysRoleRDao.findByHqlWhere(roleWhere);

        if(roleList != null && roleList.size() > 0){
            List<String> moduleListRdd = new ArrayList<>();
            List<String> featuresListRdd = new ArrayList<>();           // 冗余
            List<SysRefRoleMenu> refs = new ArrayList<>();                 // 角色权限关系

            // 封装需要删除的IDS
            Long[] rids = new Long[roleList.size()];
            for (int i = 0; i < roleList.size(); i++) {
                rids[i] = roleList.get(i).getId();
            }
            // 删除角色权限关系
            String deleteWhere = CoreHqlPack.packHqlIds("roleId", roleIds);
            List<SysRefRoleMenu> refRoleMenuList = refRoleMenuRDao.findByHqlWhere(deleteWhere);
            refRoleMenuRDao.deleteAll(refRoleMenuList);

            if(ids != null && ids.length > 0){
                String menuWhere = CoreHqlPack.packHqlIds("id", ids);
                List<SysMenu> list = sysMenuRDao.findByHqlWhere(menuWhere);
                SysRefRoleMenu ref = null;

                // 插入角色权限关系
                if(list != null && list.size() > 0){
                    for (SysMenu menu : list) {
                        for (SysRole role : roleList) {
                            ref = new SysRefRoleMenu();
                            ref.setRoleId(role.getId());
                            ref.setRoleNameRdd(role.getName());
                            ref.setMenuNameRdd(menu.getName());
                            ref.setMenuId(menu.getId());
                            ref.setMenuTypeRdd(menu.getMenuType());

                            ConvertBean.serviceModelInitialize(ref,userName);

                            refs.add(ref);
                        }

                        // 冗余信息封装
                        if(CommonEnumUtil.METHOD_LEVEL.MODULE.getValue() == menu.getMenuType().intValue()){
                            moduleListRdd.add(menu.getName());
                        }
                        if(CommonEnumUtil.METHOD_LEVEL.METHOD.getValue() == menu.getMenuType().intValue()){
                            featuresListRdd.add(menu.getName());
                        }
                    }
                }
            }

            // 冗余信息
            for (SysRole role : roleList) {
                role.setModuleNumber(moduleListRdd.size());
                role.setModuleNamesRdd(String.join(",",moduleListRdd));

                role.setMenuNumber(featuresListRdd.size());
                role.setMenuNamesRdd(String.join(",",featuresListRdd));

                ConvertBean.serviceModelUpdate(role,userName);
            }

            refRoleMenuRDao.saveAll(refs);
            sysRoleRDao.saveAll(roleList);
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("角色信息不存在")
                    .setErrorSolution("请重新输入")
                    .build();
        }
    }

    @Override
    @ApiOperation(value = "查信角色菜单关系",notes = "查询 ID 查询角色菜单关系")
    public List<SysRefRoleMenu> findSysRefRoleMenuByRoleId(Long roleId) {
        LOGGER.info("系统角色权限关系 RefRoleMenu find By RoleId");
        return refRoleMenuRDao.findByProperty(
                new String[]{"roleId","isValid"},
                new Object[]{roleId,CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()}
                );
    }

    @Override
    @ApiOperation(value = "查信角色信息",notes = "查询所有角色信息")
    public List<SysRole> findSysRoleAll() {
        LOGGER.info("系统角色 SYS_ROLE find All");
        return sysRoleRDao.findAll();
    }

    @Override
    @ApiOperation(value = "查信角色信息",notes = "根据 ID 查询角色信息")
    public SysRole getSysRoleById(Long  id) {
        LOGGER.info("系统角色 SYS_ROLE find id:{}", id);
        return sysRoleRDao.getOne(id);
    }

    @Override
    @ApiOperation(value = "查信角色信息",notes = "查询角色信息(分页/组合)")
    public ListPager<SysRole> querySysRole(SysRole role, Pager pager) {
        LOGGER.info("系统角色 SysRole find role :{} page :{}", role, pager);

        if (role == null) {
            //不传入实体对象，查询所有
            int count = sysRoleRDao.listCount();
            pager = PagerHelper.getPager(pager, count);
            return new ListPager(sysRoleRDao.listPager(pager), pager);
        } else {
            //生成hql查询语句
            String hqlPack = CoreHqlPack.packHqlSysRole(role);
            pager = PagerHelper.getPager(pager, sysRoleRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(sysRoleRDao.findByHqlWherePage(hqlPack + role.orderBy(), pager), pager);
        }
    }
}
